package websockets;

import io.undertow.Undertow;
import io.undertow.websockets.WebSocketConnectionCallback;
import io.undertow.websockets.core.AbstractReceiveListener;
import io.undertow.websockets.core.BufferedTextMessage;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.core.WebSockets;
import io.undertow.websockets.spi.WebSocketHttpExchange;
import tcp.CuisineTCPServer;

import static io.undertow.Handlers.path;
import static io.undertow.Handlers.websocket;
import static io.undertow.Handlers.*;

public class CuisineWebSocketServer {

    CuisineWebSocketConnectionHandler fConnectionHandler;
    CuisineWebSocketMessageHandler fMessageHandler;
    Undertow fWebSocketServer;
    String fWebSocketHost;
    int fWebSocketPort;

    public CuisineWebSocketServer(String iHost, int iPort
    ) {
        fWebSocketPort = iPort;
        fWebSocketHost = iHost;
    }

    public void initTCPServer(CuisineTCPServer iTcpServer) {
        fMessageHandler = new CuisineWebSocketMessageHandler(iTcpServer);
    }

    public void start() {
        fConnectionHandler = new CuisineWebSocketConnectionHandler(fMessageHandler);
        System.out.println("Opening web socket server at "+fWebSocketHost+":"+fWebSocketPort);
        fWebSocketServer = Undertow.builder()
                .addHttpListener(fWebSocketPort, "0.0.0.0")
                .setHandler(
                        path()
                                .addPrefixPath("/data", websocket(fConnectionHandler))
                )
                .build();
        fWebSocketServer.start();
    }


    private class CuisineWebSocketConnectionHandler implements WebSocketConnectionCallback {
        private CuisineWebSocketMessageHandler fWebSocketMessageHandler;

        protected CuisineWebSocketConnectionHandler(CuisineWebSocketMessageHandler iWebSocketMessageHandler) {
            fWebSocketMessageHandler = iWebSocketMessageHandler;
        }

        @Override
        public void onConnect(WebSocketHttpExchange exchange, WebSocketChannel channel) {
            channel.getReceiveSetter().set(fWebSocketMessageHandler);
            channel.resumeReceives();
        }
    }

    private class CuisineWebSocketMessageHandler extends AbstractReceiveListener {

        private CuisineTCPServer fTCPServer;

        protected CuisineWebSocketMessageHandler(CuisineTCPServer iTCPServer) {
            super();
            fTCPServer = iTCPServer;
        }

        protected void onFullTextMessage(WebSocketChannel iChannel, BufferedTextMessage iTextMessage) {
            //Handle message on webSocket
            WebSockets.sendText(String.valueOf(fTCPServer.getCurrentTemperature()), iChannel, null);
        }
    }

}
